-- THE BG
application:setBackgroundColor(0x00aaee)

-- SOME VARIABLES
local myappwidth, myappheight = application:getContentWidth(), application:getContentHeight()

-- THE OBJs
local objground = loadObj("gfx/ground_grass", "ground_grass.obj")
local objwell = loadObj("gfx/well", "well.coveredopen.obj")
local objplayer = loadObj("gfx/cubex01", "cubex01.obj")

-- THE OBJs DIMENSIONS
local wellminx, wellminy, wellminz, wellmaxx, wellmaxy, wellmaxz =
			objwell.min[1], objwell.min[2], objwell.min[3],
			objwell.max[1], objwell.max[2], objwell.max[3]
local wellcx, wellcy, wellcz =
			(wellminx + wellmaxx) / 2, (wellminy + wellmaxy) / 2, (wellminz + wellmaxz) / 2
local wellmx, wellmy, wellmz =
			wellmaxx - wellminx, wellmaxy - wellminy, wellmaxz - wellminz
local playerminx, playerminy, playerminz, playermaxx, playermaxy, playermaxz =
			objplayer.min[1], objplayer.min[2], objplayer.min[3],
			objplayer.max[1], objplayer.max[2], objplayer.max[3]
local playercx, playercy, playercz =
			(playerminx + playermaxx) / 2, (playerminy + playermaxy) / 2, (playerminz + playermaxz) / 2
local playermx, playermy, playermz =
			playermaxx - playerminx, playermaxy - playerminy, playermaxz - playerminz

-- THE VIEWPORT
local mysprite = Sprite.new()
local myviewport = Viewport.new()
local mymatrix = Matrix.new()
mymatrix:perspectiveProjection(30, -1, 1, 1000)
myviewport:setProjection(mymatrix)
myviewport:setPosition(myappwidth / 2, myappheight / 2)
myviewport:setScale(myappwidth, myappheight, 1)
myviewport:setContent(mysprite)
stage:addChild(myviewport)

-- THE LIGHT
local lx, ly, lz = myviewport:getTransform():transformPoint(1, 1, 1)
Lighting.setLight(lx, ly, lz, 0.5)
Lighting.setSpecular(0.3, 0.3, 0.3, 32)

-- 1 GROUND INSTANCE
local ground1 = Viewport.new()
ground1:setContent(objground)
ground1:setScale(3, 1, 3)
ground1:setPosition(0, 0, -3)
mysprite:addChild(ground1)

-- SOME WELL INSTANCES
for w = 1, 10 do
	local well = Viewport.new()
	well:setContent(objwell)
	well:setPosition(math.random(-24, 24), objground.max[2] - wellminy, math.random(-24, 24))
	mysprite:addChild(well)
end
Lighting.apply(objwell)

-- 1 PLAYER INSTANCE
local player1 = Viewport.new()
player1:setContent(objplayer)
--player1:globalToLocal(player1:getX(), player1:getY(), player1:getZ())
player1:setRotationY(270)
player1:setScale(0.7)
--player1:setPosition(0, objground.max[2] - playerminy, 4)
player1:setPosition(0, 5, 4)
mysprite:addChild(player1)
Lighting.apply(objplayer)
player1.vy = 0

-- A CAMERA
local camx, camy, camz, camrx, camry

-- SOME DEBUGGING
local mytext = TextField.new(nil, "XXX")
mytext:setScale(3)
mytext:setPosition(0, 24)
stage:addChild(mytext)

-- THE GAME
local game = {}
game.keys = {}

function update(e)
	-- gravity
	player1.vy -= 0.25 * e.deltaTime
	player1:setY(player1:getY() + player1.vy)

	if player1:getRotationY() > 360 then player1:setRotationY(0) end
	if player1:getRotationY() < 0 then player1:setRotationY(360) end

	-- ground collision
	if player1:getY() < objground.max[2] - playerminy then player1:setY(objground.max[2] - playerminy) end

	-- debugging
	mytext:setText(
		"a = "..player1:getY()..
		"\b = "..player1:getRotationY()..
		"\c = ".."N/A")

--[[
	-- 1st setup LOOKANGLES
	if game.keys[KeyCode.UP] then
		player1:setX(player1:getX() - math.cos(^< player1:getRotationY()) * 0.1)
		player1:setZ(player1:getZ() + math.sin(^< player1:getRotationY()) * 0.1)
	end
	if game.keys[KeyCode.DOWN] then
		player1:setX(player1:getX() + math.cos(^< player1:getRotationY()) * 0.1)
		player1:setZ(player1:getZ() - math.sin(^< player1:getRotationY()) * 0.1)
	end
	if game.keys[KeyCode.LEFT] then
--		player1:setRotationY(player1:getRotationY() + 1)
		if game.keys[KeyCode.UP] then
			player1:setRotationY(player1:getRotationY() + 1)
		elseif game.keys[KeyCode.DOWN] then
			player1:setRotationY(player1:getRotationY() - 1)
		else
			player1:setRotationY(player1:getRotationY() + 1)
		end
	end
	if game.keys[KeyCode.RIGHT] then
--		player1:setRotationY(player1:getRotationY() - 1)
		if game.keys[KeyCode.UP] then
			player1:setRotationY(player1:getRotationY() - 1)
		elseif game.keys[KeyCode.DOWN] then
			player1:setRotationY(player1:getRotationY() + 1)
		else
			player1:setRotationY(player1:getRotationY() - 1)
		end
	end

	-- SIDE SCROLLING STYLE
	myviewport:lookAngles(player1:getX(), player1:getY() + 16, player1:getZ() + 32, -27, 0, 0)
]]

	-- 2nd setup LOOKAT
	if game.keys[KeyCode.UP] then
		player1:setX(player1:getX() + math.cos(^< player1:getRotationY()) * 0.1)
		player1:setZ(player1:getZ() - math.sin(^< player1:getRotationY()) * 0.1)
	end
	if game.keys[KeyCode.DOWN] then
		player1:setX(player1:getX() - math.cos(^< player1:getRotationY()) * 0.1)
		player1:setZ(player1:getZ() + math.sin(^< player1:getRotationY()) * 0.1)
	end
	if game.keys[KeyCode.LEFT] then
		if game.keys[KeyCode.UP] then
			player1:setRotationY(player1:getRotationY() + 1)
		elseif game.keys[KeyCode.DOWN] then
			player1:setRotationY(player1:getRotationY() - 1)
		else
			player1:setRotationY(player1:getRotationY() + 1)
		end
	end
	if game.keys[KeyCode.RIGHT] then
		if game.keys[KeyCode.UP] then
			player1:setRotationY(player1:getRotationY() - 1)
		elseif game.keys[KeyCode.DOWN] then
			player1:setRotationY(player1:getRotationY() + 1)
		else
			player1:setRotationY(player1:getRotationY() - 1)
		end
	end
	-- JUMP
	if game.keys[KeyCode.SPACE] then
		player1.vy = 0.175
		game.keys[KeyCode.SPACE] = false
	end

	camx, camy, camz, camrx, camry =
		player1:getX(), player1:getY() + 16, player1:getZ() - 32,
		player1:getRotationX(), player1:getRotationY()

	-- RESIDENT EVIL STYLE
	myviewport:lookAt(camx, camy, camz, player1:getX(), player1:getY(), player1:getZ(),	0, 1, 0)
end

function keyDown(event)
	local key, rc = event.keyCode, event.realCode
	game.keys[key] = true
	game.keys[rc] = true
end

function keyUp(event)
	local key, rc = event.keyCode, event.realCode
	game.keys[key] = false
	game.keys[rc] = false
end

stage:addEventListener(Event.ENTER_FRAME, update)
stage:addEventListener(Event.KEY_DOWN, keyDown)
stage:addEventListener(Event.KEY_UP, keyUp)
